"""Tests for 100% coverage of dependencies module."""

from crimsy import Depends
from crimsy.dependencies import _DependsClass


async def test_depends_repr() -> None:
    """Test __repr__ method of _DependsClass."""

    async def get_value() -> int:
        return 42

    dep = _DependsClass(get_value)
    assert repr(dep) == "Depends(get_value)"


async def test_dependency_error_case() -> None:
    """Test error case when dependency is None in params."""
    from starlette.testclient import TestClient

    from crimsy import Crimsy, Router
    from crimsy.params import ParamInfo, ParamType, extract_params

    # Create a ParamInfo with dependency type but no dependency
    param = ParamInfo(
        name="test",
        annotation=int,
        default=None,
        param_type=ParamType.DEPENDENCY,
        dependency=None,
    )

    # Create a mock request
    app = Crimsy()
    router = Router(prefix="/")

    @router.get("/test")
    async def test_handler() -> dict[str, str]:
        return {"status": "ok"}

    app.add_router(router)
    client = TestClient(app)

    # Get a real request object
    with client:
        response = client.get("/test")
        # We can't directly test extract_params with a None dependency
        # because it would be caught during parameter analysis,
        # but we verify the error message would be correct
        assert response.status_code == 200

    # Test the error condition directly

    class MockRequest:
        method = "GET"

    try:
        # This should raise ValueError
        await extract_params(MockRequest(), [param], {})  # type: ignore[arg-type]
        assert False, "Should have raised ValueError"
    except ValueError as e:
        assert "Dependency not configured for parameter: test" in str(e)


async def test_async_iterator_dependency() -> None:
    """Test async iterator dependency to cover line 290."""
    import typing

    from starlette.testclient import TestClient

    from crimsy import Crimsy, Router

    async def get_async_value() -> typing.AsyncIterator[int]:
        """Async generator dependency."""
        yield 100

    app = Crimsy()
    router = Router(prefix="/")

    @router.get("/")
    async def handler(value: int = Depends(get_async_value)) -> dict[str, int]:
        return {"value": value}

    app.add_router(router)

    client = TestClient(app)
    response = client.get("/")

    assert response.status_code == 200
    assert response.json() == {"value": 100}



"""Exception classes for Crimsy."""


class HTTPException(Exception):
    """HTTP exception with status code and message.

    This exception can be raised anywhere in your application (endpoints, dependencies, etc.)
    and will be automatically handled to return an HTTP response with the specified
    status code and message.

    Example:
        @router.get("/users/{user_id}")
        async def get_user(user_id: int) -> User:
            if user_id not in database:
                raise HTTPException(status_code=404, message="User not found")
            return database[user_id]
    """

    def __init__(self, status_code: int, message: str = "") -> None:
        """Initialize HTTP exception.

        Args:
            status_code: HTTP status code to return
            message: Error message to include in response
        """
        self.status_code = status_code
        self.message = message
        super().__init__(message)

    def __repr__(self) -> str:
        """Return string representation."""
        return (
            f"HTTPException(status_code={self.status_code}, message={self.message!r})"
        )



"""Main Crimsy application class."""

from typing import Any, Callable

from starlette.applications import Starlette
from starlette.middleware import Middleware
from starlette.requests import Request
from starlette.responses import JSONResponse
from starlette.routing import Mount

from crimsy.exceptions import HTTPException
from crimsy.openapi import get_openapi_route, get_swagger_ui_route
from crimsy.router import Router


class Crimsy(Starlette):
    """Main Crimsy application class that wraps Starlette."""

    def __init__(
        self,
        *,
        debug: bool = False,
        middleware: list[Middleware] | None = None,
        title: str = "Crimsy API",
        version: str = "0.1.0",
        openapi_url: str = "/openapi.json",
        docs_url: str = "/docs",
    ) -> None:
        """Initialize the Crimsy application.

        Args:
            debug: Enable debug mode
            middleware: List of middleware to add
            title: API title for OpenAPI
            version: API version for OpenAPI
            openapi_url: URL path for OpenAPI JSON schema
            docs_url: URL path for Swagger UI documentation
        """
        self.title = title
        self.version = version
        self.openapi_url = openapi_url
        self.docs_url = docs_url
        self._routers: list[Router] = []

        routes = []

        # Add OpenAPI and docs routes
        if openapi_url:
            routes.append(get_openapi_route(self, openapi_url))
        if docs_url and openapi_url:
            routes.append(get_swagger_ui_route(self, docs_url, openapi_url))

        # Create default exception handlers
        exception_handlers = self._create_default_exception_handlers()

        super().__init__(
            debug=debug,
            routes=routes,
            middleware=middleware,
            exception_handlers=exception_handlers,
        )

    def add_router(self, router: Router) -> None:
        """Add a router to the application.

        Args:
            router: Router instance to add
        """
        self._routers.append(router)
        # Mount the router's routes
        mount = Mount(router.prefix, routes=router.routes)
        self.routes.append(mount)

    def _create_default_exception_handlers(
        self,
    ) -> dict[type[Exception] | int, Callable[..., Any]]:
        """Create default exception handlers for the application."""

        async def http_exception_handler(
            request: Request, exc: HTTPException
        ) -> JSONResponse:
            """Handle HTTPException by returning JSON response with status code and message."""
            return JSONResponse(
                content={"error": exc.message} if exc.message else {},
                status_code=exc.status_code,
            )

        async def generic_exception_handler(
            request: Request, exc: Exception
        ) -> JSONResponse:
            """Handle generic exceptions by returning 500 with error message."""
            return JSONResponse(
                content={"error": str(exc)},
                status_code=500,
            )

        return {
            HTTPException: http_exception_handler,
            Exception: generic_exception_handler,
        }

    def exception_handler(  # type: ignore[override]
        self, exc_class: type[Exception]
    ) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
        """Decorator to register an exception handler.

        Args:
            exc_class: Exception class to handle

        Returns:
            Decorator function

        Example:
            @app.exception_handler(CustomException)
            async def handler(request, exc):
                raise HTTPException(status_code=503, message="Service unavailable")
        """

        def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
            # Register with Starlette
            self.add_exception_handler(exc_class, func)
            return func

        return decorator

    def openapi_schema(self) -> dict[str, Any]:
        """Generate the OpenAPI schema for the application.

        Returns:
            OpenAPI schema dictionary
        """
        from crimsy.openapi import generate_openapi_schema

        return generate_openapi_schema(self)



"""Tests for the Crimsy application."""

import msgspec
from starlette.testclient import TestClient

from crimsy import Crimsy, Router


class User(msgspec.Struct):
    """User model for testing."""

    name: str
    age: int = 0


async def test_basic_get_route() -> None:
    """Test basic GET route with query parameters."""
    app = Crimsy()
    router = Router(prefix="/users")

    @router.get("/")
    async def handler(name: str) -> dict[str, str]:
        return {"message": f"Hello, {name}!"}

    app.add_router(router)

    client = TestClient(app)
    response = client.get("/users/?name=Alice")

    assert response.status_code == 200
    assert response.json() == {"message": "Hello, Alice!"}


async def test_get_with_msgspec_struct() -> None:
    """Test GET route with msgspec.Struct in body."""
    app = Crimsy()
    router = Router(prefix="/users")

    @router.post("/")
    async def handler(user: User) -> User:
        return user

    app.add_router(router)

    client = TestClient(app)
    response = client.post(
        "/users/",
        json={"name": "Bob", "age": 30},
    )

    assert response.status_code == 200
    data = response.json()
    assert data["name"] == "Bob"
    assert data["age"] == 30


async def test_multiple_query_params() -> None:
    """Test route with multiple query parameters."""
    app = Crimsy()
    router = Router(prefix="/api")

    @router.get("/search")
    async def search(query: str, limit: int = 10) -> dict[str, int | str]:
        return {"query": query, "limit": limit}

    app.add_router(router)

    client = TestClient(app)
    response = client.get("/api/search?query=test&limit=20")

    assert response.status_code == 200
    assert response.json() == {"query": "test", "limit": 20}


async def test_post_route() -> None:
    """Test POST route."""
    app = Crimsy()
    router = Router(prefix="/items")

    @router.post("/create")
    async def create_item(name: str, price: float) -> dict[str, str | float]:
        return {"name": name, "price": price, "status": "created"}

    app.add_router(router)

    client = TestClient(app)
    response = client.post("/items/create?name=Widget&price=9.99")

    assert response.status_code == 200
    data = response.json()
    assert data["name"] == "Widget"
    assert data["price"] == 9.99


async def test_put_route() -> None:
    """Test PUT route."""
    app = Crimsy()
    router = Router(prefix="/items")

    @router.put("/update")
    async def update_item(item_id: int) -> dict[str, int | str]:
        return {"item_id": item_id, "status": "updated"}

    app.add_router(router)

    client = TestClient(app)
    response = client.put("/items/update?item_id=123")

    assert response.status_code == 200
    assert response.json() == {"item_id": 123, "status": "updated"}


async def test_delete_route() -> None:
    """Test DELETE route."""
    app = Crimsy()
    router = Router(prefix="/items")

    @router.delete("/remove")
    async def delete_item(item_id: int) -> dict[str, int | str]:
        return {"item_id": item_id, "status": "deleted"}

    app.add_router(router)

    client = TestClient(app)
    response = client.delete("/items/remove?item_id=456")

    assert response.status_code == 200
    assert response.json() == {"item_id": 456, "status": "deleted"}


async def test_openapi_endpoint() -> None:
    """Test that OpenAPI JSON endpoint is available."""
    app = Crimsy()
    router = Router(prefix="/users")

    @router.get("/")
    async def list_users() -> list[dict[str, str]]:
        return [{"name": "Alice"}, {"name": "Bob"}]

    app.add_router(router)

    client = TestClient(app)
    response = client.get("/openapi.json")

    assert response.status_code == 200
    schema = response.json()
    assert schema["openapi"] == "3.0.0"
    assert schema["info"]["title"] == "Crimsy API"
    assert "/users/" in schema["paths"]


async def test_swagger_ui_endpoint() -> None:
    """Test that Swagger UI endpoint is available."""
    app = Crimsy()

    client = TestClient(app)
    response = client.get("/docs")

    assert response.status_code == 200
    assert "swagger-ui" in response.text.lower()


async def test_msgspec_struct_response() -> None:
    """Test returning msgspec.Struct from endpoint."""
    app = Crimsy()
    router = Router(prefix="/users")

    @router.get("/user")
    async def get_user(name: str) -> User:
        return User(name=name, age=25)

    app.add_router(router)

    client = TestClient(app)
    response = client.get("/users/user?name=Charlie")

    assert response.status_code == 200
    data = response.json()
    assert data["name"] == "Charlie"
    assert data["age"] == 25


async def test_missing_required_param() -> None:
    """Test that missing required parameters return error."""
    app = Crimsy()
    router = Router(prefix="/api")

    @router.get("/test")
    async def test_endpoint(required_param: str) -> dict[str, str]:
        return {"param": required_param}

    app.add_router(router)

    client = TestClient(app)
    response = client.get("/api/test")

    assert response.status_code == 400
    assert "error" in response.json()


async def test_complex_struct() -> None:
    """Test complex msgspec.Struct with nested data."""

    class Address(msgspec.Struct):
        street: str
        city: str

    class Person(msgspec.Struct):
        name: str
        address: Address

    app = Crimsy()
    router = Router(prefix="/people")

    @router.post("/")
    async def create_person(person: Person) -> Person:
        return person

    app.add_router(router)

    client = TestClient(app)
    response = client.post(
        "/people/",
        json={
            "name": "David",
            "address": {"street": "123 Main St", "city": "Springfield"},
        },
    )

    assert response.status_code == 200
    data = response.json()
    assert data["name"] == "David"
    assert data["address"]["street"] == "123 Main St"
    assert data["address"]["city"] == "Springfield"


async def test_different_http_methods() -> None:
    """Test that all HTTP methods work correctly."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/")
    async def get_handler() -> dict[str, str]:
        return {"method": "GET"}

    @router.post("/")
    async def post_handler() -> dict[str, str]:
        return {"method": "POST"}

    @router.put("/")
    async def put_handler() -> dict[str, str]:
        return {"method": "PUT"}

    @router.delete("/")
    async def delete_handler() -> dict[str, str]:
        return {"method": "DELETE"}

    @router.patch("/")
    async def patch_handler() -> dict[str, str]:
        return {"method": "PATCH"}

    app.add_router(router)

    client = TestClient(app)

    assert client.get("/test/").json() == {"method": "GET"}
    assert client.post("/test/").json() == {"method": "POST"}
    assert client.put("/test/").json() == {"method": "PUT"}
    assert client.delete("/test/").json() == {"method": "DELETE"}
    assert client.patch("/test/").json() == {"method": "PATCH"}



"""OpenAPI schema generation and documentation endpoints."""

import inspect
from typing import Any

import msgspec
from starlette.requests import Request
from starlette.responses import HTMLResponse, JSONResponse
from starlette.routing import Route

from crimsy.params import ParamInfo, ParamType, is_msgspec_struct


def get_openapi_route(app: Any, path: str) -> Route:
    """Create the OpenAPI JSON endpoint.

    Args:
        app: Ethereal application instance
        path: URL path for the endpoint

    Returns:
        Starlette Route for OpenAPI JSON
    """

    async def openapi_handler(request: Request) -> JSONResponse:
        schema = app.openapi_schema()
        return JSONResponse(schema)

    return Route(path, openapi_handler, methods=["GET"])


def get_swagger_ui_route(app: Any, docs_path: str, openapi_path: str) -> Route:
    """Create the Swagger UI documentation endpoint.

    Args:
        app: Ethereal application instance
        docs_path: URL path for Swagger UI
        openapi_path: URL path for OpenAPI JSON

    Returns:
        Starlette Route for Swagger UI
    """

    async def swagger_ui_handler(request: Request) -> HTMLResponse:
        # Note: For production use, consider either:
        # 1. Bundling these assets locally
        # 2. Using correct SRI hashes from the Swagger UI distribution
        # 3. Self-hosting the Swagger UI files
        html = f"""
        <!DOCTYPE html>
        <html>
        <head>
            <title>{app.title} - Swagger UI</title>
            <link rel="stylesheet" type="text/css" href="https://unpkg.com/swagger-ui-dist@5.9.0/swagger-ui.css">
            <link rel="icon" type="image/png" href="https://unpkg.com/swagger-ui-dist@5.9.0/favicon-32x32.png" sizes="32x32" />
            <link rel="icon" type="image/png" href="https://unpkg.com/swagger-ui-dist@5.9.0/favicon-16x16.png" sizes="16x16" />
        </head>
        <body>
            <div id="swagger-ui"></div>
            <script src="https://unpkg.com/swagger-ui-dist@5.9.0/swagger-ui-bundle.js"></script>
            <script src="https://unpkg.com/swagger-ui-dist@5.9.0/swagger-ui-standalone-preset.js"></script>
            <script>
                window.onload = function() {{
                    window.ui = SwaggerUIBundle({{
                        url: "{openapi_path}",
                        dom_id: '#swagger-ui',
                        presets: [
                            SwaggerUIBundle.presets.apis,
                            SwaggerUIStandalonePreset
                        ],
                        layout: "BaseLayout",
                        deepLinking: true,
                        showExtensions: true,
                        showCommonExtensions: true
                    }});
                }};
            </script>
        </body>
        </html>
        """
        return HTMLResponse(content=html)

    return Route(docs_path, swagger_ui_handler, methods=["GET"])


def generate_openapi_schema(app: Any) -> dict[str, Any]:
    """Generate OpenAPI 3.0 schema for the application.

    Args:
        app: Crimsy application instance

    Returns:
        OpenAPI schema dictionary
    """
    paths: dict[str, Any] = {}

    # Iterate through all routers
    for router in app._routers:
        for route in router.routes:
            # Get the path with router prefix
            full_path = router.prefix.rstrip("/") + route.path

            # Get the original endpoint and parameters
            endpoint = getattr(route.endpoint, "_original_endpoint", None)
            params = getattr(route.endpoint, "_params", [])
            # Get the original methods list (before Starlette adds HEAD for GET)
            original_methods = getattr(route.endpoint, "_original_methods", None)

            if endpoint is None:
                continue

            # Use original methods if available, otherwise fallback to route.methods
            methods_to_document = (
                original_methods if original_methods else route.methods or []
            )

            # Get endpoint metadata
            operation = generate_operation(endpoint, params, methods_to_document, app)

            # Add to paths
            if full_path not in paths:
                paths[full_path] = {}

            for method in methods_to_document:
                paths[full_path][method.lower()] = operation

    schema = {
        "openapi": "3.0.0",
        "info": {
            "title": app.title,
            "version": app.version,
        },
        "paths": paths,
    }

    return schema


def generate_operation(
    endpoint: Any, params: list[ParamInfo], methods: list[str], app: Any
) -> dict[str, Any]:
    """Generate OpenAPI operation object for an endpoint.

    Args:
        endpoint: Endpoint function
        params: List of parameter information
        methods: HTTP methods
        app: Crimsy application instance

    Returns:
        OpenAPI operation dictionary
    """
    operation: dict[str, Any] = {
        "responses": {
            "200": {
                "description": "Successful response",
            }
        },
    }

    # Add summary and description from docstring
    if endpoint.__doc__:
        operation["summary"] = endpoint.__doc__.strip().split("\n")[0]
        operation["description"] = endpoint.__doc__.strip()

    # Add parameters
    parameters = []
    request_body = None

    for param in params:
        if param.param_type == ParamType.DEPENDENCY:
            # Dependencies don't show up in OpenAPI parameters
            pass
        elif param.param_type == ParamType.QUERY:
            param_schema = get_parameter_schema(param)
            parameters.append(
                {
                    "name": param.name,
                    "in": "query",
                    "required": param.is_required,
                    "schema": param_schema,
                }
            )
        elif param.param_type == ParamType.BODY:
            # Body parameters
            schema = get_parameter_schema(param)
            request_body = {
                "required": param.is_required,
                "content": {
                    "application/json": {
                        "schema": schema,
                    }
                },
            }
        elif param.param_type == ParamType.PATH:
            param_schema = get_parameter_schema(param)
            parameters.append(
                {
                    "name": param.name,
                    "in": "path",
                    "required": True,
                    "schema": param_schema,
                }
            )

    if parameters:
        operation["parameters"] = parameters

    if request_body:
        operation["requestBody"] = request_body

    # Add response schema
    sig = inspect.signature(endpoint)
    if sig.return_annotation is not inspect.Signature.empty:
        return_schema = get_type_schema(sig.return_annotation)
        operation["responses"]["200"]["content"] = {
            "application/json": {
                "schema": return_schema,
            }
        }

    return operation


def get_parameter_schema(param: ParamInfo) -> dict[str, Any]:
    """Get OpenAPI schema for a parameter.

    Args:
        param: Parameter information

    Returns:
        OpenAPI schema dictionary
    """
    return get_type_schema(param.annotation)


def get_type_schema(annotation: type[Any]) -> dict[str, Any]:
    """Get OpenAPI schema for a type annotation.

    Args:
        annotation: Type annotation

    Returns:
        OpenAPI schema dictionary
    """
    # Handle msgspec.Struct
    if is_msgspec_struct(annotation):
        return get_msgspec_struct_schema(annotation)

    # Handle basic types
    if annotation is str:
        return {"type": "string"}
    if annotation is int:
        return {"type": "integer"}
    if annotation is float:
        return {"type": "number"}
    if annotation is bool:
        return {"type": "boolean"}
    if annotation in (dict, Any):
        return {"type": "object"}
    if annotation is list:
        return {"type": "array", "items": {}}

    # Handle generic types
    origin = getattr(annotation, "__origin__", None)
    if origin is list:
        args = getattr(annotation, "__args__", ())
        if args:
            return {"type": "array", "items": get_type_schema(args[0])}
        return {"type": "array", "items": {}}

    if origin is dict:
        return {"type": "object"}

    # Default
    return {"type": "string"}


def get_msgspec_struct_schema(struct_type: type[msgspec.Struct]) -> dict[str, Any]:
    """Get OpenAPI schema for a msgspec.Struct type.

    Args:
        struct_type: msgspec.Struct type

    Returns:
        OpenAPI schema dictionary
    """
    properties: dict[str, Any] = {}
    required = []

    # Get struct fields
    for field in msgspec.structs.fields(struct_type):
        field_schema = get_type_schema(field.type)
        properties[field.name] = field_schema

        # Check if field is required (no default value)
        if (
            field.default is msgspec.NODEFAULT
            and field.default_factory is msgspec.NODEFAULT
        ):
            required.append(field.name)

    schema: dict[str, Any] = {
        "type": "object",
        "properties": properties,
    }

    if required:
        schema["required"] = required

    return schema



"""Router class for grouping endpoints."""

import inspect
from typing import Any, Callable

import msgspec
from starlette.requests import Request
from starlette.responses import JSONResponse, Response
from starlette.routing import Route

from crimsy.params import analyze_function_params, extract_params


class Router:
    """Router for grouping API endpoints."""

    def __init__(self, prefix: str = "") -> None:
        """Initialize a router.

        Args:
            prefix: URL prefix for all routes in this router
        """
        self._prefix = prefix
        self.routes: list[Route] = []

    @property
    def prefix(self) -> str:
        """Get the router prefix."""
        return self._prefix

    def add_route(
        self,
        path: str,
        endpoint: Callable[..., Any],
        methods: list[str],
    ) -> None:
        """Add a route to the router.

        Args:
            path: URL path for the route
            endpoint: Handler function
            methods: HTTP methods for the route
        """
        # Analyze function parameters based on primary HTTP method
        primary_method = methods[0] if methods else "GET"
        params = analyze_function_params(endpoint, primary_method, path)

        # Create wrapper that handles parameter extraction and response encoding
        async def route_handler(request: Request) -> Response:
            try:
                # Extract path parameters
                path_params = request.path_params

                # Extract all parameters from request
                kwargs = await extract_params(request, params, path_params)

                # Call the endpoint
                if inspect.iscoroutinefunction(endpoint):
                    result = await endpoint(**kwargs)
                else:
                    result = endpoint(**kwargs)

                # Encode response
                if isinstance(result, Response):
                    return result

                # Use msgspec to encode the result
                if result is None:
                    return Response(status_code=204)

                json_bytes = msgspec.json.encode(result)
                return Response(
                    content=json_bytes,
                    media_type="application/json",
                )

            except ValueError as e:
                # ValueError becomes 400 Bad Request
                return JSONResponse(
                    content={"error": str(e)},
                    status_code=400,
                )
            # Let all other exceptions propagate to Starlette's exception handlers

        route = Route(path, route_handler, methods=methods)
        # Store metadata for OpenAPI generation
        route.endpoint._original_endpoint = endpoint  # type: ignore[attr-defined]
        route.endpoint._params = params  # type: ignore[attr-defined]
        route.endpoint._original_methods = methods  # type: ignore[attr-defined]
        self.routes.append(route)

    def get(self, path: str) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
        """Decorator to add a GET route.

        Args:
            path: URL path for the route

        Returns:
            Decorator function
        """

        def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
            self.add_route(path, func, ["GET"])
            return func

        return decorator

    def post(self, path: str) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
        """Decorator to add a POST route.

        Args:
            path: URL path for the route

        Returns:
            Decorator function
        """

        def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
            self.add_route(path, func, ["POST"])
            return func

        return decorator

    def put(self, path: str) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
        """Decorator to add a PUT route.

        Args:
            path: URL path for the route

        Returns:
            Decorator function
        """

        def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
            self.add_route(path, func, ["PUT"])
            return func

        return decorator

    def delete(self, path: str) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
        """Decorator to add a DELETE route.

        Args:
            path: URL path for the route

        Returns:
            Decorator function
        """

        def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
            self.add_route(path, func, ["DELETE"])
            return func

        return decorator

    def patch(self, path: str) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
        """Decorator to add a PATCH route.

        Args:
            path: URL path for the route

        Returns:
            Decorator function
        """

        def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
            self.add_route(path, func, ["PATCH"])
            return func

        return decorator

    def head(self, path: str) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
        """Decorator to add a HEAD route.

        Args:
            path: URL path for the route

        Returns:
            Decorator function
        """

        def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
            self.add_route(path, func, ["HEAD"])
            return func

        return decorator

    def options(self, path: str) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
        """Decorator to add an OPTIONS route.

        Args:
            path: URL path for the route

        Returns:
            Decorator function
        """

        def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
            self.add_route(path, func, ["OPTIONS"])
            return func

        return decorator



"""Tests to achieve 100% code coverage."""

import msgspec
from starlette.responses import Response
from starlette.testclient import TestClient

from crimsy import Body, Crimsy, Path, Router


class User(msgspec.Struct):
    """User model for testing."""

    name: str
    age: int = 25


class Item(msgspec.Struct):
    """Item model for testing."""

    title: str
    price: float


async def test_sync_handler() -> None:
    """Test synchronous (non-async) handler function."""
    app = Crimsy()
    router = Router(prefix="/sync")

    @router.get("/")
    def sync_handler(name: str) -> dict[str, str]:  # Not async
        return {"message": f"Hello, {name}!"}

    app.add_router(router)

    client = TestClient(app)
    response = client.get("/sync/?name=World")

    assert response.status_code == 200
    assert response.json() == {"message": "Hello, World!"}


async def test_response_return() -> None:
    """Test handler that returns a Starlette Response directly."""
    app = Crimsy()
    router = Router(prefix="/custom")

    @router.get("/")
    async def custom_response() -> Response:
        return Response(content=b"Custom response", media_type="text/plain")

    app.add_router(router)

    client = TestClient(app)
    response = client.get("/custom/")

    assert response.status_code == 200
    assert response.text == "Custom response"


async def test_none_response() -> None:
    """Test handler that returns None (204 No Content)."""
    app = Crimsy()
    router = Router(prefix="/empty")

    @router.delete("/{item_id}")
    async def delete_item(item_id: int = Path()) -> None:
        return None

    app.add_router(router)

    client = TestClient(app)
    response = client.delete("/empty/123")

    assert response.status_code == 204


async def test_generic_exception_handling() -> None:
    """Test that generic exceptions return 500 status code."""
    app = Crimsy()
    router = Router(prefix="/error")

    @router.get("/")
    async def raise_error() -> dict[str, str]:
        raise RuntimeError("Something went wrong")

    app.add_router(router)

    client = TestClient(app, raise_server_exceptions=False)
    response = client.get("/error/")

    assert response.status_code == 500
    assert "error" in response.json()


async def test_head_and_options_routes() -> None:
    """Test HEAD and OPTIONS HTTP methods."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.head("/")
    async def head_handler() -> dict[str, str]:
        return {"method": "HEAD"}

    @router.options("/")
    async def options_handler() -> dict[str, str]:
        return {"method": "OPTIONS"}

    app.add_router(router)

    client = TestClient(app)

    head_response = client.head("/test/")
    assert head_response.status_code == 200

    options_response = client.options("/test/")
    assert options_response.status_code == 200
    assert options_response.json() == {"method": "OPTIONS"}


async def test_handler_without_docstring() -> None:
    """Test handler without docstring for OpenAPI generation."""
    app = Crimsy()
    router = Router(prefix="/nodoc")

    @router.get("/")
    async def no_doc_handler(name: str) -> dict[str, str]:
        # No docstring
        return {"name": name}

    app.add_router(router)

    client = TestClient(app)
    response = client.get("/nodoc/?name=test")
    assert response.status_code == 200


async def test_path_parameters() -> None:
    """Test path parameter extraction and OpenAPI generation."""
    app = Crimsy()
    router = Router(prefix="/items")

    @router.get("/{item_id}")
    async def get_item(item_id: int = Path()) -> dict[str, int]:
        """Get an item by ID."""
        return {"item_id": item_id}

    app.add_router(router)

    client = TestClient(app)
    response = client.get("/items/456")
    assert response.status_code == 200
    assert response.json() == {"item_id": 456}

    # Check OpenAPI includes path parameter
    openapi_response = client.get("/openapi.json")
    schema = openapi_response.json()
    assert "/items/{item_id}" in schema["paths"]


async def test_missing_path_parameter() -> None:
    """Test error when required path parameter is missing."""
    app = Crimsy()
    router = Router(prefix="/items")

    @router.get("/{item_id}")
    async def get_item(item_id: int = Path()) -> dict[str, int]:
        return {"item_id": item_id}

    app.add_router(router)

    # This would actually be handled by Starlette routing,
    # but we test the parameter extraction logic
    client = TestClient(app)
    response = client.get("/items/abc")  # Invalid int
    assert response.status_code == 400


async def test_invalid_json_in_query_param() -> None:
    """Test invalid JSON for msgspec.Struct in query parameter."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/")
    async def handler(user: User) -> User:
        return user

    app.add_router(router)

    client = TestClient(app)
    # Invalid JSON
    response = client.get("/test/?user=not_valid_json")
    assert response.status_code == 400
    assert "error" in response.json()


async def test_missing_required_body_parameter() -> None:
    """Test missing required body parameter."""
    app = Crimsy()
    router = Router(prefix="/users")

    @router.post("/")
    async def create_user(user: User = Body()) -> User:
        return user

    app.add_router(router)

    client = TestClient(app)
    # Empty body
    response = client.post("/users/", json=None)
    assert response.status_code == 400


async def test_body_parameter_with_default() -> None:
    """Test body parameter with default value."""
    app = Crimsy()
    router = Router(prefix="/test")

    default_user = User(name="Default", age=0)

    @router.post("/")
    async def handler(user: User = Body(default=default_user)) -> User:
        return user

    app.add_router(router)

    client = TestClient(app)
    # Empty body should use default
    response = client.post("/test/", content=b"")
    assert response.status_code == 200
    assert response.json()["name"] == "Default"


async def test_invalid_body_json() -> None:
    """Test invalid JSON in request body."""
    app = Crimsy()
    router = Router(prefix="/users")

    @router.post("/")
    async def create_user(user: User = Body()) -> User:
        return user

    app.add_router(router)

    client = TestClient(app)
    # Invalid JSON
    response = client.post(
        "/users/",
        content=b"not valid json",
        headers={"content-type": "application/json"},
    )
    assert response.status_code == 400


async def test_non_struct_body_parameter() -> None:
    """Test non-msgspec.Struct body parameter."""
    app = Crimsy()
    router = Router(prefix="/data")

    @router.post("/")
    async def process_data(data: dict[str, str] = Body()) -> dict[str, str]:
        return {"received": str(data)}

    app.add_router(router)

    client = TestClient(app)
    response = client.post("/data/", json={"key": "value"})
    assert response.status_code == 200


async def test_parameter_without_annotation() -> None:
    """Test parameter without type annotation."""
    from crimsy.params import analyze_function_params

    # Create a function with no type annotation
    def handler(value):  # type: ignore[no-untyped-def]
        return {"value": str(value)}

    # Analyze it - should default to str
    params = analyze_function_params(handler, "GET")
    assert len(params) == 1
    assert params[0].annotation is str  # Should default to str


async def test_invalid_int_query_param() -> None:
    """Test invalid integer value in query parameter."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/")
    async def handler(count: int) -> dict[str, int]:
        return {"count": count}

    app.add_router(router)

    client = TestClient(app)
    response = client.get("/test/?count=not_a_number")
    assert response.status_code == 400


async def test_invalid_float_query_param() -> None:
    """Test invalid float value in query parameter."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/")
    async def handler(price: float) -> dict[str, float]:
        return {"price": price}

    app.add_router(router)

    client = TestClient(app)
    response = client.get("/test/?price=not_a_float")
    assert response.status_code == 400


async def test_bool_query_param() -> None:
    """Test boolean query parameter conversion."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/")
    async def handler(active: bool) -> dict[str, bool]:
        return {"active": active}

    app.add_router(router)

    client = TestClient(app)

    # Test various true values
    for value in ["true", "True", "1", "yes"]:
        response = client.get(f"/test/?active={value}")
        assert response.status_code == 200
        assert response.json()["active"] is True

    # Test false value
    response = client.get("/test/?active=false")
    assert response.status_code == 200
    assert response.json()["active"] is False


async def test_complex_type_query_param() -> None:
    """Test complex type in query parameter (decoded as JSON)."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/")
    async def handler(data: list[str]) -> dict[str, list[str]]:
        return {"data": data}

    app.add_router(router)

    client = TestClient(app)
    import json

    data_json = json.dumps(["a", "b", "c"])
    response = client.get(f"/test/?data={data_json}")
    assert response.status_code == 200
    assert response.json()["data"] == ["a", "b", "c"]


async def test_invalid_complex_type_query_param() -> None:
    """Test invalid JSON for complex type in query parameter."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/")
    async def handler(data: list[str]) -> dict[str, list[str]]:
        return {"data": data}

    app.add_router(router)

    client = TestClient(app)
    response = client.get("/test/?data=not_valid_json")
    assert response.status_code == 400


async def test_openapi_with_various_types() -> None:
    """Test OpenAPI schema generation with various parameter types."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/types")
    async def handler(
        text: str,
        number: int,
        decimal: float,
        flag: bool,
        items: list[str],
        data: dict[str, str],
    ) -> dict[str, str]:
        """Handler with various types."""
        return {"status": "ok"}

    app.add_router(router)

    client = TestClient(app)
    response = client.get("/openapi.json")
    schema = response.json()

    # Check that all parameter types are documented
    params = schema["paths"]["/test/types"]["get"]["parameters"]
    param_names = [p["name"] for p in params]
    assert "text" in param_names
    assert "number" in param_names
    assert "decimal" in param_names
    assert "flag" in param_names


async def test_openapi_with_generic_list() -> None:
    """Test OpenAPI schema generation with generic list type."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/")
    async def handler() -> list[Item]:
        """Return a list of items."""
        return [Item(title="Test", price=10.0)]

    app.add_router(router)

    client = TestClient(app)
    response = client.get("/openapi.json")
    schema = response.json()

    # Check response schema
    assert "/test/" in schema["paths"]


async def test_openapi_with_plain_list() -> None:
    """Test OpenAPI schema generation with plain list type."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/")
    async def handler() -> list[int]:  #
        """Return a plain list."""
        return [1, 2, 3]

    app.add_router(router)

    client = TestClient(app)
    response = client.get("/openapi.json")
    schema = response.json()

    # Check that it generates schema
    assert "/test/" in schema["paths"]


async def test_openapi_with_dict_type() -> None:
    """Test OpenAPI schema generation with dict type."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/")
    async def handler() -> dict[str, int]:
        """Return a dict."""
        return {"count": 5}

    app.add_router(router)

    client = TestClient(app)
    response = client.get("/openapi.json")
    schema = response.json()

    # Check that it generates schema
    assert "/test/" in schema["paths"]


async def test_param_marker_repr() -> None:
    """Test __repr__ method of parameter markers."""
    from crimsy.params import Query, Body, Path

    # Test without default
    q1 = Query()
    assert repr(q1) == "Query()"

    # Test with default
    q2 = Query(default="test")
    assert repr(q2) == "Query(default='test')"

    b1 = Body()
    assert repr(b1) == "Body()"

    b2 = Body(default={"key": "value"})
    assert repr(b2) == "Body(default={'key': 'value'})"

    p1 = Path()
    assert repr(p1) == "Path()"

    p2 = Path(default=0)
    assert repr(p2) == "Path(default=0)"


async def test_is_msgspec_struct_with_non_type() -> None:
    """Test is_msgspec_struct with non-type values."""
    from crimsy.params import is_msgspec_struct

    # Test with non-type value
    assert is_msgspec_struct("not a type") is False  # type: ignore[arg-type]
    assert is_msgspec_struct(123) is False  # type: ignore[arg-type]
    assert is_msgspec_struct(None) is False  # type: ignore[arg-type]

    # Test with actual struct
    assert is_msgspec_struct(User) is True


async def test_class_method_parameters_skipped() -> None:
    """Test that self and cls parameters are skipped in analysis."""
    from crimsy.params import analyze_function_params

    class Handler:
        def method(self, name: str) -> dict[str, str]:
            return {"name": name}

    # Analyze the method - 'self' should be skipped
    params = analyze_function_params(Handler.method, "GET")
    # Should only have 'name' parameter, not 'self'
    param_names = [p.name for p in params]
    assert "self" not in param_names
    assert "name" in param_names


async def test_openapi_endpoint_without_original_endpoint() -> None:
    """Test OpenAPI generation skips routes without _original_endpoint."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/")
    async def handler() -> dict[str, str]:
        return {"status": "ok"}

    # Manually remove the _original_endpoint attribute
    if router.routes:
        delattr(router.routes[0].endpoint, "_original_endpoint")

    app.add_router(router)

    client = TestClient(app)
    response = client.get("/openapi.json")
    # Should not crash, just skip the route
    assert response.status_code == 200


async def test_openapi_with_any_type() -> None:
    """Test OpenAPI schema generation with Any type."""
    from typing import Any

    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/")
    async def handler() -> Any:
        """Return Any type."""
        return {"anything": "goes"}

    app.add_router(router)

    client = TestClient(app)
    response = client.get("/openapi.json")
    schema = response.json()
    # Should generate schema without error
    assert "/test/" in schema["paths"]


async def test_openapi_with_unknown_type() -> None:
    """Test OpenAPI schema generation with unknown type (fallback to string)."""
    app = Crimsy()
    router = Router(prefix="/test")

    class CustomClass:
        pass

    @router.get("/")
    async def handler() -> CustomClass:
        """Return custom class."""
        return CustomClass()

    app.add_router(router)

    client = TestClient(app)
    response = client.get("/openapi.json")
    schema = response.json()
    # Should generate schema with default 'string' type
    assert "/test/" in schema["paths"]


async def test_openapi_with_list_without_args() -> None:
    """Test OpenAPI schema generation with generic list without type args."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/")
    async def handler(items: list[str]) -> dict[str, str]:
        """Handler with plain list parameter."""
        return {"status": "ok"}

    app.add_router(router)

    client = TestClient(app)
    response = client.get("/openapi.json")
    schema = response.json()
    # Should handle list without args
    assert "/test/" in schema["paths"]


async def test_get_request_with_missing_struct_in_query() -> None:
    """Test GET request with msgspec.Struct but missing query parameter."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/")
    async def handler(user: User) -> User:
        return user

    app.add_router(router)

    client = TestClient(app)
    # No user parameter provided - should try to read from body and fail
    response = client.get("/test/")
    assert response.status_code == 400


async def test_path_parameter_with_default() -> None:
    """Test path parameter with default value (edge case)."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/{item_id}")
    async def handler(
        item_id: int = Path(default=0),
    ) -> dict[str, int]:
        return {"item_id": item_id}

    app.add_router(router)

    client = TestClient(app)
    # Normal case with path param
    response = client.get("/test/123")
    assert response.status_code == 200
    assert response.json()["item_id"] == 123


async def test_is_msgspec_struct_typeerror() -> None:
    """Test is_msgspec_struct handles TypeError."""
    from crimsy.params import is_msgspec_struct
    from typing import Union

    # Test with type that causes TypeError in issubclass
    result = is_msgspec_struct(Union[str, int])  # type: ignore[arg-type]
    assert result is False


async def test_get_request_with_invalid_struct_json_in_query() -> None:
    """Test GET request with msgspec.Struct and invalid JSON in query."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/")
    async def handler(user: User) -> User:
        return user

    app.add_router(router)

    client = TestClient(app)
    # Invalid JSON in user query parameter for GET request
    response = client.get("/test/?user={bad json}")
    assert response.status_code == 400
    assert "error" in response.json()


async def test_missing_required_path_parameter_edge_case() -> None:
    """Test extraction logic for missing path parameter."""
    from crimsy.params import ParamInfo, ParamType, extract_params
    from starlette.datastructures import QueryParams
    import inspect

    # Create a mock request
    class MockRequest:
        method = "GET"
        query_params = QueryParams({})

        async def body(self) -> bytes:
            return b""

    # Create param info for required path parameter
    param = ParamInfo(
        name="missing_id",
        annotation=int,
        default=inspect.Parameter.empty,
        param_type=ParamType.PATH,
    )

    request = MockRequest()
    path_params: dict[str, str] = {}  # Empty path params

    # Should raise ValueError for missing required path parameter
    try:
        await extract_params(request, [param], path_params)  # type: ignore[arg-type]
        assert False, "Should have raised ValueError"
    except ValueError as e:
        assert "Missing required path parameter" in str(e)


async def test_path_parameter_with_default_missing() -> None:
    """Test path parameter with default when path param is missing."""
    from crimsy.params import ParamInfo, ParamType, extract_params
    from starlette.datastructures import QueryParams

    # Create a mock request
    class MockRequest:
        method = "GET"
        query_params = QueryParams({})

        async def body(self) -> bytes:
            return b""

    # Create param info for path parameter with default
    param = ParamInfo(
        name="item_id",
        annotation=int,
        default=99,  # Has default
        param_type=ParamType.PATH,
    )

    request = MockRequest()
    path_params: dict[str, str] = {}  # Empty path params

    # Should use default value
    result = await extract_params(request, [param], path_params)  # type: ignore[arg-type]
    assert result["item_id"] == 99


async def test_openapi_generic_list_without_args_edge_case() -> None:
    """Test OpenAPI schema for list type without __args__."""
    from crimsy.openapi import get_type_schema

    # Create a type that has __origin__ as list but no __args__
    class FakeListType:
        __origin__ = list

    result = get_type_schema(FakeListType())  # type: ignore[arg-type]
    # Should return array with empty items
    assert result["type"] == "array"
    assert "items" in result


async def test_is_msgspec_struct_with_generic_causing_typeerror() -> None:
    """Test is_msgspec_struct with type that causes TypeError in issubclass."""
    from crimsy.params import is_msgspec_struct
    from typing import Generic, TypeVar

    T = TypeVar("T")

    class GenericClass(Generic[T]):
        pass

    # This should trigger the TypeError exception handler
    result = is_msgspec_struct(GenericClass[str])
    assert result is False


async def test_get_request_body_struct_with_empty_query() -> None:
    """Test GET request with struct parameter when query param is empty/missing."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/process")
    async def handler(user: User) -> User:
        """Process user data."""
        return user

    app.add_router(router)

    client = TestClient(app)
    # GET request without user in query - should try body and fail
    response = client.get("/test/process")
    assert response.status_code == 400
    assert "error" in response.json()


async def test_get_request_body_struct_query_empty_string() -> None:
    """Test GET request with struct parameter as empty string in query."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/process")
    async def handler(user: User) -> User:
        """Process user data."""
        return user

    app.add_router(router)

    client = TestClient(app)
    # GET request with empty user query param - should skip to body
    response = client.get("/test/process?user=")
    assert response.status_code == 400


async def test_head_request_body_struct() -> None:
    """Test HEAD request with msgspec.Struct parameter."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.head("/check")
    async def handler(user: User) -> User:
        """Check user."""
        return user

    app.add_router(router)

    client = TestClient(app)
    # HEAD request should also try query params for struct
    import json

    user_json = json.dumps({"name": "Alice", "age": 30})
    response = client.head(f"/test/check?user={user_json}")
    assert response.status_code == 200


async def test_options_request_body_struct() -> None:
    """Test OPTIONS request with msgspec.Struct parameter."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.options("/info")
    async def handler(user: User) -> User:
        """Get info."""
        return user

    app.add_router(router)

    client = TestClient(app)
    # OPTIONS request should also try query params for struct
    import json

    user_json = json.dumps({"name": "Bob", "age": 25})
    response = client.options(f"/test/info?user={user_json}")
    assert response.status_code == 200


async def test_openapi_plain_list_type() -> None:
    """Test OpenAPI schema generation with plain list type (not list[T])."""
    from crimsy.openapi import get_type_schema

    # Test with plain list type
    result = get_type_schema(list)
    assert result["type"] == "array"
    assert "items" in result


async def test_is_msgspec_struct_with_union_type() -> None:
    """Test is_msgspec_struct with Union type that causes TypeError."""
    from crimsy.params import is_msgspec_struct
    from typing import Union

    # Union types cause TypeError in issubclass check
    result = is_msgspec_struct(Union[str, int])  # type: ignore[arg-type]
    assert result is False


async def test_get_request_struct_without_query_value() -> None:
    """Test GET request with msgspec.Struct when query param value is missing."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/process")
    async def handler(user: User) -> User:
        """Process user."""
        return user

    app.add_router(router)

    client = TestClient(app)
    # GET request without user query param - should fail looking for body
    response = client.get("/test/process")
    assert response.status_code == 400
    assert "error" in response.json()


async def test_head_request_struct_without_query_value() -> None:
    """Test HEAD request with msgspec.Struct when query param value is missing."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.head("/check")
    async def handler(user: User) -> User:
        """Check user."""
        return user

    app.add_router(router)

    client = TestClient(app)
    # HEAD request without user query param - should fail
    response = client.head("/test/check")
    assert response.status_code == 400


async def test_options_request_struct_without_query_value() -> None:
    """Test OPTIONS request with msgspec.Struct when query param value is missing."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.options("/info")
    async def handler(user: User) -> User:
        """Get info."""
        return user

    app.add_router(router)

    client = TestClient(app)
    # OPTIONS request without user query param - should fail
    response = client.options("/test/info")
    assert response.status_code == 400


async def test_get_struct_query_param_explicit_coverage() -> None:
    """Explicit test to cover GET request with struct in query param - lines 249-257."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/handler")
    async def handler(data: User) -> User:
        """Handler with struct parameter."""
        return data

    app.add_router(router)

    client = TestClient(app)

    # Test with valid JSON in query param
    import json

    user_data = json.dumps({"name": "TestUser", "age": 42})
    response = client.get(f"/test/handler?data={user_data}")

    assert response.status_code == 200
    result = response.json()
    assert result["name"] == "TestUser"
    assert result["age"] == 42


async def test_get_struct_query_param_invalid_json_coverage() -> None:
    """Test GET request with invalid JSON for struct in query param - exception path."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/handler")
    async def handler(data: User) -> User:
        """Handler with struct parameter."""
        return data

    app.add_router(router)

    client = TestClient(app)

    # Test with invalid JSON in query param - should trigger DecodeError exception
    response = client.get("/test/handler?data={invalid json}")

    assert response.status_code == 400
    assert "error" in response.json()


async def test_get_with_body_marker_on_struct() -> None:
    """Test GET request with msgspec.Struct explicitly marked as Body().

    This covers the special case in lines 245-259 where a struct is marked
    as Body but the request is GET/HEAD/OPTIONS, so we try query params first.
    """
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/search")
    async def search(
        user: User = Body(),
    ) -> dict[str, str]:
        """Search with user data from body (or query for GET)."""
        return {"found": user.name}

    app.add_router(router)

    client = TestClient(app)

    # For GET with Body() marker, it should try query param first
    import json

    user_json = json.dumps({"name": "QueryUser", "age": 30})
    response = client.get(f"/test/search?user={user_json}")

    assert response.status_code == 200
    assert response.json()["found"] == "QueryUser"


async def test_get_with_body_marker_invalid_json() -> None:
    """Test GET request with Body() marker and invalid JSON in query param."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.get("/search")
    async def search(
        user: User = Body(),
    ) -> dict[str, str]:
        """Search with user data."""
        return {"found": user.name}

    app.add_router(router)

    client = TestClient(app)

    # Invalid JSON should trigger DecodeError in lines 256-259
    response = client.get("/test/search?user={not valid json}")

    assert response.status_code == 400
    assert "error" in response.json()


async def test_head_with_body_marker_on_struct() -> None:
    """Test HEAD request with Body() marker on struct."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.head("/check")
    async def check(
        user: User = Body(),
    ) -> dict[str, str]:
        """Check user."""
        return {"status": "ok"}

    app.add_router(router)

    client = TestClient(app)

    import json

    user_json = json.dumps({"name": "HeadUser", "age": 25})
    response = client.head(f"/test/check?user={user_json}")

    assert response.status_code == 200


async def test_options_with_body_marker_on_struct() -> None:
    """Test OPTIONS request with Body() marker on struct."""
    app = Crimsy()
    router = Router(prefix="/test")

    @router.options("/info")
    async def info(
        user: User = Body(),
    ) -> dict[str, str]:
        """Get info."""
        return {"info": "data"}

    app.add_router(router)

    client = TestClient(app)

    import json

    user_json = json.dumps({"name": "OptionsUser", "age": 20})
    response = client.options(f"/test/info?user={user_json}")

    assert response.status_code == 200


async def test_is_msgspec_struct_typeerror_with_mock() -> None:
    """Test is_msgspec_struct TypeError exception handler using mock."""
    from unittest.mock import patch
    from crimsy.params import is_msgspec_struct

    # Create a type-like object
    class FakeType(type):
        pass

    fake_cls = FakeType("FakeCls", (), {})

    # Mock issubclass to raise TypeError
    with patch("crimsy.params.issubclass", side_effect=TypeError("mock error")):
        result = is_msgspec_struct(fake_cls)
        assert result is False



"""Dependency injection system for Crimsy."""

from collections.abc import AsyncIterator, Awaitable
from typing import Any, Callable, TypeVar, overload

T = TypeVar("T")


class _DependsClass:
    """Internal class for dependency injection marker."""

    dependency: Callable[..., Any]
    use_cache: bool

    def __init__(self, dependency: Callable[..., Any], use_cache: bool = True) -> None:
        """Initialize dependency marker.

        Args:
            dependency: Callable that provides the dependency
            use_cache: Whether to cache the dependency result (default: True)
        """
        self.dependency = dependency
        self.use_cache = use_cache

    def __repr__(self) -> str:
        """Return string representation."""
        return f"Depends({self.dependency.__name__})"


# Overloads for async iterators (unwrap AsyncIterator[T] to T)
@overload
def Depends(
    dependency: Callable[..., AsyncIterator[T]], use_cache: bool = True
) -> T: ...


# Overloads for async functions returning T (unwrap Awaitable[T] to T)
@overload
def Depends(dependency: Callable[..., Awaitable[T]], use_cache: bool = True) -> T: ...


# Overloads for sync functions returning T
@overload
def Depends(dependency: Callable[..., T], use_cache: bool = True) -> T: ...


def Depends(dependency: Callable[..., Any], use_cache: bool = True) -> Any:
    """Marker for dependency injection.

    Similar to FastAPI's Depends, this function marks a parameter as a dependency
    that should be resolved and injected by the framework.

    The function signature is designed so that type checkers understand that
    Depends(func) returns the same type as func's return type (unwrapping
    Awaitable and AsyncIterator for async functions).

    Example:
        async def get_db() -> Database:
            return Database()

        @router.get("/")
        async def handler(db: Database = Depends(get_db)) -> dict:
            return {"status": "ok"}
    """
    return _DependsClass(dependency, use_cache)



"""Crimsy - A lightweight web framework for building APIs in Python."""

from crimsy.app import Crimsy
from crimsy.dependencies import Depends
from crimsy.exceptions import HTTPException
from crimsy.params import Body, Path, Query
from crimsy.router import Router

__all__ = [
    "Crimsy",
    "Router",
    "Query",
    "Body",
    "Path",
    "Depends",
    "HTTPException",
]
